==> gbk-2.9.tar.gz: gbk-2.9/gbook-secure.c <==
/*
	A setuid wrapper to secure web bulletin board config file and directory
 
	Users need to run this program before they 
	want to use web bulletin board CGI.

*/

/* $Id$ */

#include "gbook.h"

int main(int argc, char *argv[])
{
	char		home[80], gbook_path[80], talk_path[80], gbook_conf[80];
	int		bid = 0;
	uid_t 		uid, euid;
	gid_t		httpd_gid;
	struct passwd   *pw;
	struct group	*grp;
	struct stat	sb;

	if ((euid = geteuid()) != 0) {
		printf("error: this program should be owned by root and setuid\n");
		
		exit(1);
	} else {

		uid = getuid();
		if (!(pw = getpwuid(uid))) {
			printf("error: getpw fail\n");
			exit(1);
		}
		if (!(grp = getgrnam(HTTPD_GRP))) {
			printf("error: can not get group id for %s\n", HTTPD_GRP);
			exit(1);
		}
		if (argc > 1) {
			bid = atoi(argv[1]);
		}
		//printf("bid=%d\n",bid);
		httpd_gid = grp->gr_gid;
		strncpy(home, pw->pw_dir, 80);
		if (bid) {
			snprintf(gbook_path, 80, "%s/%s%d", home, GBOOK_DIR, bid);
		} else {
			snprintf(gbook_path, 80, "%s/%s", home, GBOOK_DIR);
		}
		strncpy(gbook_conf, gbook_path, 80);
		strncat(gbook_conf, "/", 80);
		strncat(gbook_conf, GBOOK_CONF, 80);
		strncpy(talk_path, gbook_path, 80);
		strncat(talk_path, "/", 80);
		strncat(talk_path, TALK_DIR, 80);

		if (!lstat(gbook_path, &sb)) {
			if ((sb.st_mode & 0777000) != S_IFDIR) {
				printf("error: your %s is not a directory\n", gbook_path);
				exit(1);
			}
		} else {
			printf("error: can not lstat() %s, or first time to create it\n", gbook_path);
			exit(1);
		}

		if (!lstat(talk_path, &sb)) {
			if ((sb.st_mode & 0777000) != S_IFDIR) {
				printf("error: your %s is not a directory\n", talk_path);
				exit(1);
			}
		} else {
			printf("error: can not lstat() %s, or first time to crea
te it\n", talk_path);
			exit(1);
		}

		if (!lstat(gbook_conf, &sb)) {
			if ((sb.st_mode & 0777000) != S_IFREG) {
				printf("error: your %s is not a regular file\n", gbook_conf);
				exit(1);
			}
		} else {
			printf("error: can not lstat() %s, or first time to crea
te it\n", gbook_conf);
			exit(1);
		}

		/* setuid 0 */

		if (setuid(0) != 0) {
			printf("error: setuid 0 fail\n");
			exit(1);
		} else {
			chown(gbook_path, uid, httpd_gid);
			chmod(gbook_path, 0770);
			chown(talk_path, uid, httpd_gid);
			chmod(talk_path, 0770);
			//printf("talkpath=%s\n",talk_path);
			chown(gbook_conf, uid, httpd_gid);
			chmod(gbook_conf, 0770);
		}
	}

	return 0;
}

==> gbk-3.92.tar.gz: gbk-3.92/gbook-secure.c <==
/*
	A setuid wrapper to secure web bulletin board config file and directory
 
	Users need to run this program before they 
	want to use web bulletin board CGI.

*/

/* $Id$ */

#include "gbook.h"

int main(int argc, char *argv[])
{
	char		home[80], gbook_path[80], talk_path[80], gbook_conf[80];
	int		bid = 0;
	uid_t 		uid, euid;
	gid_t		httpd_gid;
	struct passwd   *pw;
	struct group	*grp;
	struct stat	sb;

	if ((euid = geteuid()) != 0) {
		printf("error: this program should be owned by root and setuid\n");
		
		exit(1);
	} else {

		uid = getuid();
		if (!(pw = getpwuid(uid))) {
			printf("error: getpw fail\n");
			exit(1);
		}
		if (!(grp = getgrnam(HTTPD_GRP))) {
			printf("error: can not get group id for %s\n", HTTPD_GRP);
			exit(1);
		}
		if (argc > 1) {
			bid = atoi(argv[1]);
		}
		//printf("bid=%d\n",bid);
		httpd_gid = grp->gr_gid;
		strncpy(home, pw->pw_dir, 80);
		if (bid) {
			snprintf(gbook_path, 80, "%s/%s%d", home, GBOOK_DIR, bid);
		} else {
			snprintf(gbook_path, 80, "%s/%s", home, GBOOK_DIR);
		}
		strncpy(gbook_conf, gbook_path, 80);
		strncat(gbook_conf, "/", 80);
		strncat(gbook_conf, GBOOK_CONF, 80);
		strncpy(talk_path, gbook_path, 80);
		strncat(talk_path, "/", 80);
		strncat(talk_path, TALK_DIR, 80);

		if (!lstat(gbook_path, &sb)) {
			if ((sb.st_mode & 0777000) != S_IFDIR) {
				printf("error: your %s is not a directory\n", gbook_path);
				exit(1);
			}
		} else {
			printf("error: can not lstat() %s, or first time to create it\n", gbook_path);
			exit(1);
		}

		if (!lstat(talk_path, &sb)) {
			if ((sb.st_mode & 0777000) != S_IFDIR) {
				printf("error: your %s is not a directory\n", talk_path);
				exit(1);
			}
		} else {
			printf("error: can not lstat() %s, or first time to create it\n", talk_path);
			exit(1);
		}

		if (!lstat(gbook_conf, &sb)) {
			if ((sb.st_mode & 0777000) != S_IFREG) {
				printf("error: your %s is not a regular file\n", gbook_conf);
				exit(1);
			}
		} else {
			printf("error: can not lstat() %s, or first time to create it\n", gbook_conf);
			exit(1);
		}

		/* setuid 0 */

		if (setuid(0) != 0) {
			printf("error: setuid 0 fail\n");
			exit(1);
		} else {
			chown(gbook_path, uid, httpd_gid);
			chmod(gbook_path, 0770);
			chown(talk_path, uid, httpd_gid);
			chmod(talk_path, 0770);
			//printf("talkpath=%s\n",talk_path);
			chown(gbook_conf, uid, httpd_gid);
			chmod(gbook_conf, 0770);
		}
	}

	return 0;
}
